@@ -22,7 +22,10 @@ from qctrlcommons.node.registry import (
 from qctrl import toolkit
 from qctrl.utils import PackageRegistry
 
-from .async_result import AsyncResult
+from .async_result import (
+    AsyncResult,
+    AsyncResultCollection,
+)
 from .client_builder import (
     create_client_auth,
     create_gql_client,
