@@ -45,6 +45,7 @@ class AsyncResult:
     PENDING = "PENDING"
     STARTED = "STARTED"
     SUCCESS = "SUCCESS"
+    REVOKED = "REVOKED"
 
     def __init__(self, field_type: GraphQLType, result_object: ResultMixin):
         self._field_type = field_type
@@ -89,6 +90,9 @@ class AsyncResult:
         if status == self.SUCCESS:
             return f"Your task {action} has completed."
 
+        if status == self.REVOKED:
+            return f"You task {action} has been revoked."
+
         return None
 
 
@@ -187,6 +191,15 @@ class AsyncResultCollection:
                     )
                     exception = "Unknown failure"
 
+                elif async_result.result_object.status == "REVOKED":
+                    LOGGER.info(
+                        "Action %s resulted with status 'REVOKED'("
+                        "no extra details about the errors are "
+                        "available).",
+                        async_result.result_object.action_id,
+                    )
+                    exception = "Unknown failure"
+
                 failures.append((identifier, exception))
 
         if not failures:
