@@ -19,7 +19,6 @@ from typing import (
 )
 from urllib.parse import urljoin
 
-from gql import Client
 from qctrlcommons.auth import ClientAuthBase
 from qctrlcommons.exceptions import QctrlException
 from qctrlcommons.utils import generate_user_agent
@@ -28,10 +27,7 @@ from qctrl.builders.http_transport import (
     QctrlGqlClient,
     QctrlRequestsHTTPTransport,
 )
-from qctrl.constants import (
-    OIDC_CLIENT_ID,
-    ORGANIZATION_SLUG_HEADER,
-)
+from qctrl.constants import OIDC_CLIENT_ID
 from qctrl.qctrlauth.session import QctrlOAuth2Session
 from qctrl.utils import (
     check_client_version,
@@ -117,8 +113,7 @@ def create_gql_client(
     api_root: str,
     auth: Optional[ClientAuthBase] = None,
     headers: Optional[Dict] = None,
-    organization_slug: Optional[str] = None,
-) -> Client:
+) -> QctrlGqlClient:
     """Creates a gql client.
 
     Parameters
@@ -129,8 +124,6 @@ def create_gql_client(
         The host url.
     headers : Dict
         HTTP headers.
-    organization_slug : str, optional
-        Organization slug for single tenant allocation.
 
     Returns
     -------
@@ -142,9 +135,6 @@ def create_gql_client(
     LOGGER.debug("gql url: %s", url)
     headers = headers or {}
 
-    if organization_slug:
-        headers[ORGANIZATION_SLUG_HEADER] = organization_slug
-
     headers.update(
         {
             "Content-Encoding": "gzip",
