@@ -12,7 +12,10 @@
 # License for the specific language.
 # pylint:disable=missing-module-docstring
 import logging
-from functools import lru_cache
+from functools import (
+    lru_cache,
+    wraps,
+)
 from typing import (
     Any,
     List,
@@ -22,8 +25,10 @@ from typing import (
 import inflection
 from gql import Client
 from graphql import GraphQLType
+from qctrlcommons.exceptions import QctrlGqlException
 from tqdm.auto import tqdm
 
+from ..mutations import UpdateActionMutation
 from ..queries import (
     MultiRefreshActionQuery,
     RefreshActionQuery,
@@ -55,6 +60,48 @@ from .verbosity import VerbosityEnum
 from .wait import Wait
 
 LOGGER = logging.getLogger(__name__)
+ACTION_READY_STATES = ["SUCCESS", "FAILURE", "REVOKED"]
+
+
+def revoke_calculation(func):
+    """Decorator for wait_for_completion() function, that revokes
+    the user's calculation when the user issues a KeyboardInterrupt."""
+
+    @wraps(func)
+    def revoke_action(qctrl_gql_environment, _, *async_results, **kwargs):
+        try:
+            func(qctrl_gql_environment, _, *async_results, **kwargs)
+        except KeyboardInterrupt:
+            promise = AsyncResultCollection(*async_results)
+            task_failure_flag = False
+            # pylint:disable=protected-access
+            update_action_mutation = UpdateActionMutation(
+                qctrl_gql_environment._gql_client
+            )
+            for (
+                async_result
+            ) in promise._async_results:  # pylint:disable=protected-access
+                action_name = async_result.result_object.name
+                action_id = async_result.result_object.action_id
+                action_status = async_result.result_object.status
+                if action_status not in ACTION_READY_STATES:
+                    try:
+                        update_action_mutation(action_id, "REVOKED")
+                        print(
+                            f'Your task {action_name} (action_id="{action_id}") has '
+                            + "been cancelled successfully."
+                        )
+                    except QctrlGqlException as exc:
+                        LOGGER.debug("mutation execution failed: %s", exc)
+                        task_failure_flag = True
+            if task_failure_flag:
+                print(
+                    "One or more tasks could not be cancelled."
+                    + " Check activity monitor for more details."
+                )
+            raise
+
+    return revoke_action
 
 
 class QctrlGraphQLEnvironment(GraphQLEnvironment):
@@ -106,6 +153,7 @@ class QctrlGraphQLEnvironment(GraphQLEnvironment):
 
         return query
 
+    @revoke_calculation
     def wait_for_completion(
         self,
         refresh_query: Union[RefreshActionQuery, MultiRefreshActionQuery],
@@ -116,27 +164,30 @@ class QctrlGraphQLEnvironment(GraphQLEnvironment):
         promise = AsyncResultCollection(*async_results)
         if promise.is_completed:
             promise.finalize()
-            return
-
-        _wait = Wait()
 
         hide_progress_bar = verbosity == VerbosityEnum.QUIET
         with tqdm(total=100, leave=False, disable=hide_progress_bar) as progress_bar:
-
-            while not promise.is_completed:
-
-                promise.update_progress(progress_bar)
-
-                _wait()
-
-                # check completion
-                response = refresh_query(*promise.get_action_ids())
-                promise.update_data(self, response, progress_bar, verbosity)
-
+            self.pull_results(promise, refresh_query, progress_bar, verbosity)
             progress_bar.update(max(0, progress_bar.total - progress_bar.n))
-
         promise.finalize()
 
+    def pull_results(
+        self,
+        promise: AsyncResultCollection,
+        refresh_query: Union[RefreshActionQuery, MultiRefreshActionQuery],
+        progress_bar: tqdm,
+        verbosity: VerbosityEnum,
+    ):
+        """Pulls the result of the calculation."""
+        _wait = Wait()
+        while not promise.is_completed:
+            promise.update_progress(progress_bar)
+            _wait()
+            # check completion
+            response = refresh_query(*promise.get_action_ids())
+            promise.update_data(self, response, progress_bar, verbosity)
+        return promise
+
     def build_refresh_query(self, field_type: GraphQLType) -> RefreshActionQuery:
         """Returns a RefreshActionQuery object for
         refreshing an object populated with response data.
