@@ -14,7 +14,6 @@
 import logging
 import os
 import sys
-import warnings
 import webbrowser
 from threading import Thread
 from typing import (
@@ -46,6 +45,7 @@ from .builders import (
 from .cite import cite_boulder_opal
 from .constants import (
     BASE_URL,
+    DEFAULT_ACCOUNTS_URL,
     DEFAULT_API_ROOT,
     DEFAULT_OIDC_URL,
     INVALID_SUBSCRIPTION_ERROR,
@@ -57,23 +57,17 @@ from .queries import (
     ActivityMonitorQuery,
     GetMutationNameQuery,
     MeQuery,
+    TenantsQuery,
 )
 from .utils import (
     PackageRegistry,
     _get_mutation_result_type,
+    print_available_organizations,
+    set_organization_client_headers,
 )
 
 LOGGER = logging.getLogger(__name__)
 
-warnings.formatwarning = lambda message, *args, **kwargs: f"{message}\n"
-if sys.version_info.minor < 8:
-    warnings.warn(
-        "You are currently using Python 3.7. "
-        "Boulder Opal will soon remove support for Python 3.7 "
-        "since it will reach its end-of-life status on June 26th, 2023.",
-        FutureWarning,
-    )
-
 
 class Qctrl:
     """A mediator class. Used to authenticate with Q-CTRL and access Q-CTRL features.
@@ -183,8 +177,10 @@ class Qctrl:
             self.gql_api = _build_client(
                 api_root=api_root,
                 oidc_url=oidc_url,
-                organization_slug=self.organization_slug,
             )
+
+            self._set_organization()
+
         else:
             if env_namespace or oidc_url or api_root:
                 raise ValueError(
@@ -397,6 +393,75 @@ class Qctrl:
         """
         self.verbosity = parse_verbosity(verbosity)
 
+    def _set_organization(self):
+        """
+        Sets the organization client headers based on default selection or user selection.
+        """
+        tenants = TenantsQuery(self.gql_api)()
+        available_tenants = {}
+        for tenant in tenants:
+            lowercase_tenant_slug = tenant["organizationSlug"].lower()
+            available_tenants[lowercase_tenant_slug] = {
+                "slug": tenant["organizationSlug"],
+                "id": tenant["organizationId"],
+            }
+        try:
+            validated_tenant = self._validate_organization_selection(available_tenants)
+        except ValueError as exc:
+            click.secho(str(exc), err=True, fg="red")
+            click.echo(
+                (
+                    f"Available organizations are listed below. "
+                    f"You can check organization names and other details on {DEFAULT_ACCOUNTS_URL}"
+                )
+            )
+            print_available_organizations(tenants)
+            sys.exit(1)
+
+        if validated_tenant:
+            set_organization_client_headers(
+                client=self.gql_api,
+                org_id=validated_tenant["id"],
+                org_slug=validated_tenant["slug"],
+            )
+
+    def _validate_organization_selection(
+        self, available_tenants
+    ) -> Optional[Dict[str, str]]:
+        """
+        Validates that an appropriate org is selected for or by the user.
+
+        Raises
+        ------
+        ValueError
+            If selected organization value is invalid.
+        """
+        if not self.organization_slug:
+
+            # use multi-tenant env if no tenants for the user
+            if len(available_tenants) == 0:
+                LOGGER.debug("No tenants were found. Using multi-tenant environment.")
+                return None
+
+            # select the org for the user if user only has one tenant
+            if len(available_tenants) == 1:
+                tenant = list(available_tenants.values())[0]
+                return tenant
+
+            raise ValueError(
+                "You are in multiple organizations."
+                + " Specify the computing environment using organization slug."
+            )
+
+        selected_organization_slug = self.organization_slug.lower()
+
+        # check chosen org is in the list of available tenants
+        if selected_organization_slug not in available_tenants:
+            raise ValueError("Selected organization slug not found.")
+
+        tenant = available_tenants.get(selected_organization_slug)
+        return tenant
+
 
 @tenacity.retry(
     wait=tenacity.wait_exponential(multiplier=1, min=1, max=5),
@@ -404,7 +469,8 @@ class Qctrl:
     retry=tenacity.retry_if_exception_type((RequestException, BaseHTTPError)),
 )
 def _build_client(
-    api_root: str, oidc_url: str, organization_slug: Optional[str] = None
+    api_root: str,
+    oidc_url: str,
 ) -> Client:
     """
     Builds an authenticated GraphQL client and validates its authentication.
@@ -426,7 +492,7 @@ def _build_client(
 
     try:
         auth = create_client_auth(oidc_url)
-        client = create_gql_client(api_root, auth, organization_slug=organization_slug)
+        client = create_gql_client(api_root, auth)
         MeQuery(client)()
 
     except QctrlException as error:
@@ -444,7 +510,6 @@ def _build_client(
             sys.exit(1)
 
         sys.exit(error)
-
     return client
 
 
