@@ -21,13 +21,15 @@ from pathlib import Path
 from typing import (
     Any,
     Callable,
+    Dict,
+    List,
     Optional,
     Union,
 )
 
 import commonmark
 import requests
-import toml
+import tomli
 from gql.transport.exceptions import TransportServerError
 from graphql import (
     GraphQLField,
@@ -42,8 +44,13 @@ from qctrlcommons.exceptions import QctrlException
 from requests import codes
 from requests.exceptions import HTTPError
 from rich.console import Console
+from rich.table import Table
 
-from qctrl.constants import UNIVERSAL_ERROR_MESSAGE
+from qctrl.constants import (
+    ORGANIZATION_ID_HEADER,
+    ORGANIZATION_SLUG_HEADER,
+    UNIVERSAL_ERROR_MESSAGE,
+)
 
 LOGGER = logging.getLogger(__name__)
 
@@ -358,11 +365,9 @@ def print_environment_related_packages():
 
         # otherwise it's in the editable mode, looking for pyproject.toml to get the package name
         with open(
-            Path.joinpath(Path(module.__path__[0]).parent, "pyproject.toml"),
-            mode="r",
-            encoding="utf-8",
+            Path.joinpath(Path(module.__path__[0]).parent, "pyproject.toml"), "rb"
         ) as file:
-            config = toml.load(file)
+            config = tomli.load(file)
             return config["tool"]["poetry"]["name"]
 
     top_level_module_names = [
@@ -377,6 +382,7 @@ def print_environment_related_packages():
         # Q-CTRL packages.
         "qctrl",
         "qctrlcommons",
+        "qctrlexperimentscheduler",
         "qctrlmloop",
         "qctrlopencontrols",
         "qctrlqua",
@@ -412,3 +418,35 @@ def print_environment_related_packages():
     # Print table.
     for name, version_ in table_items:
         print(f"| {name:{package_width}s} | {version_:{version_width}s} |")
+
+
+def update_client_headers(client, headers):
+    """
+    Updates headers with any additional headers.
+    """
+    client.transport.headers.update(headers)
+
+
+def set_organization_client_headers(client, org_id: str, org_slug: str):
+    """
+    Sets the organizations related headers on the client transport.
+    """
+    org_headers = {
+        ORGANIZATION_ID_HEADER: org_id,
+        ORGANIZATION_SLUG_HEADER: org_slug,
+    }
+    update_client_headers(client, org_headers)
+
+
+def print_available_organizations(organization_data: List[Dict[str, str]]):
+    """
+    Prints a formatted response using the result of the tenants query.
+    """
+    tenant_slugs = [tenant["organizationSlug"] for tenant in organization_data]
+    table = Table(title="Your Organizations", show_lines=True)
+    console = Console(force_jupyter=False)
+    table.add_column("No.", overflow="fold")
+    table.add_column("Organization Slug", overflow="fold")
+    for idx, tenant_slug in enumerate(tenant_slugs):
+        table.add_row(str(idx + 1), tenant_slug)
+    console.print(table)
